@@ -3,7 +3,7 @@
 **
 ** <file/class description>
 **
-** Copyright (C) 2002-2005 Steve Lhomme.  All rights reserved.
+** Copyright (C) 2002-2010 Steve Lhomme.  All rights reserved.
 **
 ** This file is part of libebml.
 **
@@ -11,12 +11,12 @@
 ** modify it under the terms of the GNU Lesser General Public
 ** License as published by the Free Software Foundation; either
 ** version 2.1 of the License, or (at your option) any later version.
-** 
+**
 ** This library is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 ** Lesser General Public License for more details.
-** 
+**
 ** You should have received a copy of the GNU Lesser General Public
 ** License along with this library; if not, write to the Free Software
 ** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -49,7 +49,7 @@ EbmlFloat::EbmlFloat(const EbmlFloat::Precision prec)
 EbmlFloat::EbmlFloat(const double aDefaultValue, const EbmlFloat::Precision prec)
  :EbmlElement(0, true), Value(aDefaultValue), DefaultValue(aDefaultValue)
 {
-	DefaultIsSet = true;
+	SetDefaultIsSet();
 	SetPrecision(prec);
 }
 
@@ -60,69 +60,91 @@ EbmlFloat::EbmlFloat(const EbmlFloat & ElementToClone)
 {
 }
 
+void EbmlFloat::SetDefaultValue(double aValue)
+{
+    assert(!DefaultISset());
+    DefaultValue = aValue;
+    SetDefaultIsSet();
+}
+
+const double EbmlFloat::DefaultVal() const
+{
+    assert(DefaultISset());
+    return DefaultValue;
+}
+
+
 /*!
 	\todo handle exception on errors
 	\todo handle 10 bits precision
 */
-uint32 EbmlFloat::RenderData(IOCallback & output, bool bForceRender, bool bKeepIntact)
+filepos_t EbmlFloat::RenderData(IOCallback & output, bool bForceRender, bool bWithDefault)
 {
-	assert(Size == 4 || Size == 8);
+	assert(GetSize() == 4 || GetSize() == 8);
 
-	if (Size == 4) {
+	if (GetSize() == 4) {
 		float val = Value;
 		int Tmp;
 		memcpy(&Tmp, &val, 4);
 		big_int32 TmpToWrite(Tmp);
-		output.writeFully(&TmpToWrite.endian(), Size);
-	} else if (Size == 8) {
+		output.writeFully(&TmpToWrite.endian(), GetSize());
+	} else if (GetSize() == 8) {
 		double val = Value;
 		int64 Tmp;
 		memcpy(&Tmp, &val, 8);
 		big_int64 TmpToWrite(Tmp);
-		output.writeFully(&TmpToWrite.endian(), Size);
-	} 
+		output.writeFully(&TmpToWrite.endian(), GetSize());
+	}
 
-	return Size;
+	return GetSize();
 }
 
-uint64 EbmlFloat::UpdateSize(bool bKeepIntact, bool bForceRender)
+uint64 EbmlFloat::UpdateSize(bool bWithDefault, bool bForceRender)
 {
-	if (!bKeepIntact && IsDefaultValue())
+	if (!bWithDefault && IsDefaultValue())
 		return 0;
-	return Size;
+	return GetSize();
 }
 
 /*!
 	\todo remove the hack for possible endianess pb (test on little & big endian)
 */
-uint64 EbmlFloat::ReadData(IOCallback & input, ScopeMode ReadFully)
+filepos_t EbmlFloat::ReadData(IOCallback & input, ScopeMode ReadFully)
 {
 	if (ReadFully != SCOPE_NO_DATA)
 	{
 		binary Buffer[20];
-		assert(Size <= 20);
-		input.readFully(Buffer, Size);
-		
-		if (Size == 4) {
+		assert(GetSize() <= 20);
+		input.readFully(Buffer, GetSize());
+
+		if (GetSize() == 4) {
 			big_int32 TmpRead;
 			TmpRead.Eval(Buffer);
 			int32 tmpp = int32(TmpRead);
 			float val;
 			memcpy(&val, &tmpp, 4);
 			Value = val;
-			bValueIsSet = true;
-		} else if (Size == 8) {
+			SetValueIsSet();
+		} else if (GetSize() == 8) {
 			big_int64 TmpRead;
 			TmpRead.Eval(Buffer);
 			int64 tmpp = int64(TmpRead);
 			double val;
 			memcpy(&val, &tmpp, 8);
 			Value = val;
-			bValueIsSet = true;
+			SetValueIsSet();
 		}
 	}
 
-	return Size;
+	return GetSize();
+}
+
+bool EbmlFloat::IsSmallerThan(const EbmlElement *Cmp) const
+{
+	if (EbmlId(*this) == EbmlId(*Cmp))
+		return this->Value < static_cast<const EbmlFloat *>(Cmp)->Value;
+	else
+		return false;
 }
 
 END_LIBEBML_NAMESPACE
